library(quantreg)
library(glmnet)

mae = function(v1, v2)
   mean(abs(v1 - v2))

mse = function(v1, v2)
   mean((v1 - v2)**2)

lasso.lambda = .001
  # We lasso-regularize our quantile regression just a little bit
  # to avoid issues with singular design matrices.

quantile.reg = function(X, y)
   {model = quantreg::rq(y ~ ., data = X,
        tau = .5, method = "lasso", lambda = lasso.lambda)
    y.pred = predict(model)
    list(
        error = mae(y, y.pred),
        coef = coef(model))}

logistic.reg = function(X, y)
   {model = glm(y ~ ., data = X,
        family = binomial(link = "logit"))
    y.pred = predict(model, type = "response")
    list(
        error = mse(y, y.pred),
        coef = coef(model),
        counterfactual.preds =  sapply(1 : ncol(X), function(ci)
          # For each predictor, calculate the mean probability of
          # the outcome among all subjects with the predictor
          # forced to be 1.
           {X.changed = X
            X.changed[,ci] = 1
            mean(predict(model, type = "response", newdata = X.changed))}))}

