(require [kodhy.macros [rmap amap filt λ qw getl geti $ wc ss ssi cbind ecase]])

(import
  pickle
  [datetime [datetime]]
  [numpy :as np]
  [pandas :as pd]
  [kodhy.util [T F R-get R-call R-assign R-run valcounts pairs seq ret hzscore pds-from-pairs]])

(print "Running")
(setv start-time (datetime.now))

(setv min-cell-size 11) ; Set by a data-use agreement.
(setv mm_cat-codes {"Medicare only" 1 "Medicaid only" 2 "Dual" 3})
(setv sas-data-path "O:/Desmond/Jennifer/Comorbidities/mcare_alldiag.sas7bdat")

(setv comorbs (qw Congestive_heart_failure Cardiac_arrhythmias Valvular_disease Peripheral_vascular_disorders Hypertension__uncomplicated Hypertension__complicated Paralysis Other_neurological_disorders Pulmonary_circulation_disorders Chronic_pulmonary_disease Diabetes__uncomplicated Diabetes__complicated Hypothyroidism Renal_failure Liver_disease Peptic_ulcer_disease Lymphoma Metastatic_cancer Solid_tumor_without_metastasis Rheumatoid_arthritis Coagulopathy Coagulopathy_hemophilia Blood_loss_anemia Deficiency_anemia Obesity Weight_loss Fluid_and_electrolyte_disorders))

(setv orig (pd.read-sas :format "sas7bdat" sas-data-path))

(R-call "source" "//tsclient/luskin/Comorbid-public/pred_support.R")

(setv sb (do
  (setv df orig)
  (assert (.all (= ($ df partd_mos) 12)))
    ; Among Medicare beneficiaries, we want only those with Part D
    ; coverage for the full year.
  (assert (.all (.isin ($ df mm_cat) [
    (get mm_cat-codes "Medicare only")
    (get mm_cat-codes "Dual")])))
    ; And we only want people with Medicare.
  (setv df (wc df (cbind
    :has_medicaid (.isin $mm_cat [
      (get mm_cat-codes "Medicaid only")
      (get mm_cat-codes "Dual")])
    :female (= $mm_sex 2)
    :age_raw $mm_age
    ; The reference group for race is white.
    :race_black (= $mm_race 2)
    :race_hispanic (= $mm_race 3)
    :race_other_nonwhite (= $mm_race 4)
    :urban $mcare_urban
    :hiv_docvol_50plus (= $docvol 1)
    :disabled (= $disabled 1)
    (.rename (= (getl df : comorbs) 1) :columns (λ (+ "cm_" it)))
    :y_outpatient $otspend
    :y_inpatient $ipspend
    :y_drugs $rxspend)))
  (setv df (.astype df float))
  (assert (.all (.all (np.isfinite df))))
  df))
(setv ($ sb age_std) (hzscore ($ sb age_raw)))
(setv ($ sb age_std2) (hzscore (** ($ sb age_std) 2)))

(setv x-full (getl sb : (filt
  (and
    (!= it "age_raw")
    (not (.startswith it "y_")))
  sb)))
(R-assign "x.full" (.as-matrix x-full))
(R-run "x.full = data.frame(x.full)")
(R-assign "colnames(x.full)" (np.array x-full.columns))
(for [yv (qw y_outpatient y_drugs y_inpatient)]
  (R-assign yv (. (getl sb : yv) values)))

(setv x-types (qw trivial demog demog+cm))

;(setv results-cv {})
;(setv results-cv (rmap [[yv x-type twin] (product (qw y_outpatient y_drugs) x-types [F T])]
;  (print yv x-type twin)
;  (, yv x-type (if twin "twin" "single") (R-call "try.pred"
;    [:raw "x.full"] [:raw yv] x-type twin))))
;(.extend results-cv (rmap [[x-type-prob x-type-amount] (product x-types x-types)]
;  (setv yv "y_inpatient")
;  (print yv x-type-prob x-type-amount)
;  (, yv x-type-prob x-type-amount (R-call "try.pred.2stage"
;    [:raw "x.full"] [:raw yv]
;    x-type-prob x-type-amount))))
;(with [o (open "//tsclient/luskin/pred_cv.pkl" "wb")]
;  (pickle.dump results-cv o))

;(print "Getting coefficients")
;(setv results-coefci {
;  "y_outpatient" (do (print "y_outpatient") (R-get
;    "quantreg.log.lambda(the.lambda, x.full, y_outpatient, coefci = TRUE)"))
;  "y_drugs" (do (print "y_drugs") (R-get
;    "quantreg.log.lambda(the.lambda, x.full, y_drugs, coefci = TRUE)"))
;  "y_inpatient_prob" (do [] (print "y_inpatient_prob")
;    (R-run "
;       library(MASS)
;       m = glm(y ~ .,
;           data = cbind(x.full, y = y_inpatient != 0),
;           family = binomial)
;       d = cbind(coef(m), confint(m, level = .95))")
;    (R-get "d"))
;  "y_inpatient_amount" ((fn [] (print "y_inpatient_amount") (R-get
;    "quantreg.log.lambda(the.lambda, x.full[y_inpatient != 0,], y_inpatient[y_inpatient != 0], coefci = TRUE)")))})
;(for [[k v] (list (.items results-coefci))]
;  (setv v (pd.DataFrame v))
;  (setv v.columns (qw estimate lo hi))
;  (setv v.index (+ ["Intercept"] (list x-full.columns)))
;  (setv (get results-coefci k) v))
;(with [o (open "//tsclient/luskin/pred_coefci.pkl" "wb")]
;  (pickle.dump results-coefci o))

(print "Took" (- (datetime.now) start-time))
